package jumpstart.min.business.domain.security;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import jumpstart.min.business.commons.exception.ServiceException;
import jumpstart.min.business.domain.base.BaseSvc;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcRemote;

//@SecurityDomain("jumpstart")
@Stateless
@Local(ISecurityManagerSvcLocal.class)
@Remote(ISecurityManagerSvcRemote.class)
public class SecurityManagerSvc extends BaseSvc implements ISecurityManagerSvcLocal, ISecurityManagerSvcRemote {

	/***************************************************************************
	 * BUSINESS METHODS
	 **************************************************************************/

	/**
	 * This method provides a way for users to change their own password.
	 */
	public void changeUserPassword(Long id, String currentPassword, String newPassword) throws ServiceException {
		User user = (User) find(User.class, id);
		user.changePassword(currentPassword, newPassword);
	}

	/**
	 * This method provides a way for security officers to "reset" the password.
	 */
	public void changeUserPassword(Long id, String newPassword) throws ServiceException {
		User user = (User) find(User.class, id);
		user.setPassword(newPassword);
	}

	public User createUser(User user, String password) throws ServiceException {
		user.changePassword(null, password);
		persist(user);
		return user;
	}

	public void changeUser(User user) throws ServiceException {
		merge(user);
	}

	public void deleteUser(User user) throws ServiceException {
		user = (User) merge(user);
		remove(user);
	}

}
